@@ -30,7 +30,7 @@ static int delay_timer = 0;
 
 int fix_crappy_cgi_headers(t_cgi_info *cgi_info) {
 	char *header_end, *new_buffer, *src, *dst;
-	int count = 0, header_len;
+	int count = 0, header_len, new_size;
 
 	if ((header_end = strstr(cgi_info->input_buffer, "\n\n")) == NULL) {
 		return 1;
@@ -49,10 +49,8 @@ int fix_crappy_cgi_headers(t_cgi_info *cgi_info) {
 
 	/* Allocate new buffer
 	 */
-	if (cgi_info->input_len + count > cgi_info->input_buffer_size) {
-		cgi_info->input_buffer_size = cgi_info->input_len + count;
-	}
-	if ((new_buffer = (char*)malloc(cgi_info->input_buffer_size + 1)) == NULL) {
+	new_size = cgi_info->input_len + count;
+	if ((new_buffer = (char*)malloc(new_size + 1)) == NULL) {
 		return -1;
 	}
 
@@ -75,6 +73,7 @@ int fix_crappy_cgi_headers(t_cgi_info *cgi_info) {
 	free(cgi_info->input_buffer);
 	cgi_info->input_buffer = new_buffer;
 	cgi_info->input_len += count;
+	cgi_info->input_buffer_size = new_size;
 
 	return 0;
 }
@@ -225,17 +224,14 @@ void check_load_balancer(t_config *config) {
  * ======================
  */
 
-t_fcgi_server *fcgi_server_match(t_fcgi_server *fcgi_server, t_charlist *fastcgi, char *cgi_script) {
-	char *extension;
+t_fcgi_server *fcgi_server_match(t_fcgi_server *fcgi_server, t_charlist *fastcgi, char *extension) {
 	t_fcgi_server *fcgi;
 	int i;
 
-	if ((fastcgi == NULL) || (cgi_script == NULL)) {
+	if ((fastcgi == NULL) || (extension == NULL)) {
 		return NULL;
 	}
 
-	extension = file_extension(cgi_script);
-
 	for (i = 0; i < fastcgi->size; i++) {
 		fcgi = fcgi_server;
 		while (fcgi != NULL) {
