@@ -53,7 +53,7 @@ int fix_crappy_cgi_headers(t_cgi_info *cgi_info);
  */
 void init_load_balancer(t_fcgi_server *fcgi_server);
 t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ip_addr *client_ip);
-t_fcgi_server *fcgi_server_match(t_fcgi_server *fcgi_server, t_charlist *fastcgi, char *cgi_script);
+t_fcgi_server *fcgi_server_match(t_fcgi_server *fcgi_server, t_charlist *fastcgi, char *extension);
 t_fcgi_server *find_fcgi_server(t_fcgi_server *fcgi_server, char *id);
 void check_load_balancer(t_config *config);
 
