@@ -15,7 +15,9 @@ static int add_to_environment(t_fcgi_buffer *fcgi_buffer, char *key, char *value
 	int len;
 	size_t key_len, value_len;
 
-	if (fcgi_buffer == NULL) {
+	if ((key == NULL) || (value == NULL)) {
+		return 0;
+	} else if (fcgi_buffer == NULL) {
 		/* Normal CGI
 		 */
 		setenv(key, value, 1);
@@ -63,7 +65,9 @@ static int add_to_environment_chroot(t_session *session, t_fcgi_buffer *fcgi_buf
 	size_t ofs = 0;
 	char c;
 
-	if (session->fcgi_server != NULL) {
+	if ((key == NULL) || (value == NULL)) {
+		return 0;
+	} else if (session->fcgi_server != NULL) {
 		if (strncmp(value, session->fcgi_server->chroot, session->fcgi_server->chroot_len) == 0) {
 			c = value[session->fcgi_server->chroot_len];
 			if ((c == '/') || (c == '\0')) {
@@ -79,7 +83,9 @@ static int add_to_environment_chroot(t_session *session, t_fcgi_buffer *fcgi_buf
 static int headerfield_to_environment(t_session *session, t_fcgi_buffer *fcgi_buffer, char *key, char *envir) {
 	char *value;
 
-	if ((value = get_headerfield(key, session->headerfields)) != NULL) {
+	if ((key == NULL) || (envir == NULL)) {
+		return 0;
+	} else if ((value = get_headerfield(key, session->headerfields)) != NULL) {
 		return add_to_environment(fcgi_buffer, envir, value);
 	} else {
 		return 0;
