@@ -149,6 +149,16 @@ void touch_logfiles(t_config *config, char *dir) {
 	}
 }
 
+/* Determine the extension of the requested file
+ */
+void determine_extension(t_session *session) {
+	if ((session->extension = strrchr(session->file_on_disk, '.')) != NULL) {
+		(session->extension)++;
+	} else {
+		session->extension = session->config->no_extension_as;
+	}
+}
+
 /* Check if the requested file is a CGI program.
  */
 t_cgi_type check_target_is_cgi(t_session *session) {
@@ -159,14 +169,14 @@ t_cgi_type check_target_is_cgi(t_session *session) {
 		session->cgi_type = fastcgi;
 	} else 
 #endif
-	if ((session->fcgi_server = fcgi_server_match(session->config->fcgi_server, &(session->host->fast_cgi), session->file_on_disk)) != NULL) {
+	if ((session->fcgi_server = fcgi_server_match(session->config->fcgi_server, &(session->host->fast_cgi), session->extension)) != NULL) {
 		session->cgi_type = fastcgi;
-	} else if (in_charlist(file_extension(session->file_on_disk), &(session->config->cgi_extension))) {
+	} else if (in_charlist(session->extension, &(session->config->cgi_extension))) {
 		session->cgi_type = program;
 	} else {
 		cgi = session->config->cgi_handler;
 		while (cgi != NULL) {
-			if (in_charlist(file_extension(session->file_on_disk), &(cgi->extension))) {
+			if (in_charlist(session->extension, &(cgi->extension))) {
 				session->cgi_handler = cgi->handler;
 				session->cgi_type = script;
 				break;
@@ -574,7 +584,9 @@ int parse_request(t_session *session, int total_bytes) {
 		return 400;
 	}
 	*(str_end++) = '\0';
-	if ((session->request_uri = strdup(session->uri)) == NULL) {
+	if (strncmp(session->uri, "http://", 7) == 0) {
+		return 400;
+	} else if ((session->request_uri = strdup(session->uri)) == NULL) {
 		return -1;
 	}
 
@@ -745,56 +757,47 @@ t_access allow_client(t_session *session) {
 }
 
 int get_path_info(t_session *session) {
-	char *uri, *dot, *slash, *old_cgi_handler;
-	t_cgi_type cgi_type, old_cgi_type;
-	int ofs = 0;
+	t_fsbool is_dir;
+	char *slash;
 	int len;
 
 	if (session->alias_used) {
-		return 0;
+		return 200;
 	}
 
 	len = session->host->website_root_len;
 	if (len >= strlen(session->file_on_disk)) {
-		return -1;
+		return 500;
 	}
 
-	uri = session->file_on_disk + len;
-	if ((dot = strchr(uri, '.')) == NULL) {
-		return 0;
-	}
+	slash = session->file_on_disk + len + 1;
+	while (*slash != '\0') {
+		if (*slash == '/') {
+			*slash = '\0';
+			is_dir = is_directory(session->file_on_disk);
+			*slash = '/';
 
-	if ((slash = strchr(dot, '/')) == NULL) {
-		return 0;
-	}
-
-	old_cgi_handler = session->cgi_handler;
-	old_cgi_type = session->cgi_type;
-
-	*slash = '\0';
-	cgi_type = check_target_is_cgi(session);
-	*slash = '/';
-
-	session->cgi_handler = old_cgi_handler;
-	session->cgi_type = old_cgi_type;
-
-	if (cgi_type != no_cgi) {
-		if ((session->path_info = strdup(slash)) == NULL) {
-			return -1;
+			switch (is_dir) {
+				case error:
+					return 500;
+				case not_found:
+					return 404;
+				case no_access:
+					return 403;
+				case no:
+					if ((session->path_info = strdup(slash)) == NULL) {
+						return -1;
+					}
+					*slash = '\0';
+					return 200;
+				case yes:
+					break;
+			}
 		}
-
-		ofs = slash - uri;
-		if (ofs < session->uri_len) {
-			*(session->uri + ofs) = '\0';
-		} else {
-			return -1;
-		}
-		*slash = '\0';
-
-		return 1;
+		slash++;
 	}
 
-	return 0;
+	return 200;
 }
 
 /* Serve the client that connected to the webserver
@@ -817,12 +820,6 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
-	if (session->host->secure_url) {
-		if (strstr(session->request_uri, "%00") != NULL) {
-			return 403;
-		}
-	}
-
 	if (session->request_method == TRACE) {
 		if (session->binding->enable_trace == false) {
 			return 501;
@@ -845,6 +842,12 @@ int serve_client(t_session *session) {
 	}
 	session->host->access_time = time(NULL);
 
+	if (session->host->secure_url) {
+		if (strstr(session->request_uri, "%00") != NULL) {
+			return 403;
+		}
+	}
+
 #ifdef HAVE_TOOLKIT
 	/* URL toolkit
 	 */
@@ -967,8 +970,8 @@ int serve_client(t_session *session) {
 			break;
 		case no:
 			if ((session->request_method != PUT) && (session->host->enable_path_info)) {
-				if (get_path_info(session) == -1) {
-					return 500;
+				if ((result = get_path_info(session)) != 200) {
+					return result;
 				}
 			} else if ((search = strrchr(session->file_on_disk, '/')) != NULL) {
 				*search = '\0';
@@ -1016,6 +1019,8 @@ int serve_client(t_session *session) {
 		return 301;
 	}
 
+	determine_extension(session);
+
 	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
 		check_target_is_cgi(session);
 		if (file_not_found && (session->cgi_type != fastcgi)) {
@@ -1244,6 +1249,7 @@ void connection_handler(t_session *session) {
 	}
 
 	/* Client session ends here */
+	pthread_exit(NULL);
 }
 
 /* Signal handlers
