@@ -190,6 +190,47 @@ static short hex_to_int(char c) {
 	 return -1;
 }
 
+/* Encode a string to an URL encoded one
+ */
+static bool char_needs_encoding(char c) {
+	return (c <= 31) || (strchr(" \"#%'<>", c) != NULL) || (c >= 126);
+}
+int url_encode(char *str, char **encoded) {
+	char *c, *e;
+	int replace = 0;
+
+	c = str;
+	while (*c != '\0') {
+		if (char_needs_encoding(*c)) {
+			replace++;
+		}
+		c++;
+	}
+
+	if (replace == 0) {
+		*encoded = NULL;
+		return 0;
+	} else if ((*encoded = (char*)malloc(strlen(str) + (2 * replace) + 1)) == NULL) {
+		return -1;
+	}
+
+	c = str;
+	e = *encoded;
+	while (*c != '\0') {
+		if (char_needs_encoding(*c)) {
+			sprintf(e, "%%%02hhx", *c);
+			e += 2;
+		} else {
+			*e = *c;
+		}
+		c++;
+		e++;
+	}
+	*e = '\0';
+
+	return replace;
+}
+
 /* Decode the URL encoded characters (%XX).
  */
 void url_decode(char *str) {
@@ -510,22 +551,6 @@ bool decode_base64(char *base64) {
 	return retval;
 }
 
-char *file_extension(char *filename) {
-	char *c;
-
-	if (filename != NULL) {
-		c = filename + strlen(filename) - 1;
-		while (c >= filename) {
-			if (*c == '.') {
-				return c + 1;
-			}
-			c--;
-		}
-	}
-
-	return NULL;
-}
-
 int str_replace(char *src, char *from, char *to, char **dst) {
 	char *pos, *start;
 	int replaced = 0, len_from, len_to, len_start;
