@@ -6,18 +6,18 @@
 int  str2int(const char *str);
 char *remove_spaces(char *str);
 char *uncomment(char *str);
-char *strlower(char *line);
+char *strlower(char *str);
 int  split_string(const char *str, char **key, char **value, char c);
 int  split_configline(const char *str, char **key, char **value);
 bool valid_uri(char *uri);
-void url_decode(char *line);
-bool forbidden_chars_present(char *line);
-int  prevent_CMDi(char *line);
-int  prevent_SQLi(char *line, int length, char **newline);
-int  prevent_XSS(char *line);
+int  url_encode(char *str, char **encoded);
+void url_decode(char *str);
+bool forbidden_chars_present(char *str);
+int  prevent_CMDi(char *str);
+int  prevent_SQLi(char *str, int length, char **newstr);
+int  prevent_XSS(char *str);
 const char *http_error(int code);
 bool decode_base64(char *base64);
-char *file_extension(char *filename);
 int  str_replace(char *src, char *from, char *to, char **dst);
 bool min_strlen(char *str, int n);
 
