@@ -234,6 +234,7 @@ t_config *default_config(void) {
 		config->allowed_ciphers    = NULL;
 #endif
 		init_charlist(&(config->cgi_extension));
+		config->no_extension_as    = NULL;
 		config->total_connections  = 100;
 		config->connections_per_ip = 10;
 		config->kill_timedout_cgi  = true;
@@ -339,19 +340,17 @@ static bool valid_file(char *file) {
 }
 
 static bool valid_directory(char *dir) {
-	bool retval = false;
 	int len;
 
-	if (dir != NULL) {
-		if (*dir == '/') {
-			len = strlen(dir);
-			if (*(dir + len - 1) != '/') {
-				retval = true;
-			}
-		}
+	if (dir == NULL) {
+		return false;
+	} else if ((len = strlen(dir)) <= 1) {
+		return false;
+	} else if ((*dir == '/') && (*(dir + len - 1) != '/')) {
+		return true;
 	}
 
-	return retval;
+	return false;
 }
 
 static int parse_mode(char *line, mode_t *mode) {
@@ -809,6 +808,10 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		if ((config->mimetype_config = strdup(value)) != NULL) {
 			return true;
 		}
+	} else if (strcmp(key, "noextensionas") == 0) {
+		if ((config->no_extension_as = strdup(value)) != NULL) {
+			return true;
+		}
 	} else if (strcmp(key, "pidfile") == 0) {
 		if (*value == '/') {
 			if ((config->pidfile = strdup(value)) != NULL) {
@@ -1583,6 +1586,8 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 					config->url_toolkit = current_toolkit;
 					section = url_toolkit;
 #endif
+				} else {
+					retval = counter;
 				}
 			} else if (strcmp(key, "}") == 0) {
 				/* Section end
