@@ -85,7 +85,7 @@ int send_file(t_session *session) {
 #ifdef HAVE_COMMAND
 	increment_counter(COUNTER_FILE);
 #endif
-	session->mimetype = get_mimetype(session->file_on_disk, session->config->mimetype);
+	session->mimetype = get_mimetype(session->extension, session->config->mimetype);
 
 	handle = -1;
 	if (session->accept_gzip) {
@@ -143,7 +143,7 @@ int send_file(t_session *session) {
 				free(session->file_on_disk);
 				if ((session->file_on_disk = (char*)malloc(strlen(session->host->imgref_replacement) + 4)) != NULL) { /* + 3 for ".gz" (gzip encoding) */
 					strcpy(session->file_on_disk, session->host->imgref_replacement);
-					session->mimetype = get_mimetype(session->file_on_disk, session->config->mimetype);
+					session->mimetype = get_mimetype(session->extension, session->config->mimetype);
 				} else {
 					close(handle);
 					return 500;
@@ -686,9 +686,9 @@ int execute_cgi(t_session *session) {
 					retval = 500;
 				}
 				if (session->config->kill_timedout_cgi && (session->cgi_type != fastcgi)) {
-					if (kill(cgi_pid, 15) != -1) {
+					if (kill(cgi_pid, SIGTERM) != -1) {
 						sleep(1);
-						kill(cgi_pid, 9);
+						kill(cgi_pid, SIGKILL);
 					}
 				}
 				break;
@@ -731,7 +731,7 @@ int execute_cgi(t_session *session) {
 							/* Fix crappy CGI headers
 							 */
 							if ((result = fix_crappy_cgi_headers(&cgi_info)) == -1) {
-								retval = -1;
+								retval = 500;
 								break;
 							} else if (result == 0) {
 								new_line = strstr(cgi_info.input_buffer, "\r\n\r\n");
@@ -920,7 +920,7 @@ static int filesize2str(char *buffer, off_t fsize) {
 int show_index(t_session *session) {
 	int total_files = 0, retval = -1, len;
 	off_t total_fsize = 0;
-	char size[FILESIZE_BUFFER_SIZE], color = '0', timestr[33], *slash;
+	char size[FILESIZE_BUFFER_SIZE], color = '0', timestr[33], *slash, *link;
 	struct tm s;
 	t_filelist *filelist = NULL, *file;
 	t_keyvalue *alias;
@@ -1082,23 +1082,39 @@ int show_index(t_session *session) {
 				}
 			}
 
-			len = strlen(file->name);
-
 			if (send_chunk(session, "</td><td class=\"filename\"><a href=\"", 35) == -1) {
 				break;
-			} else if (send_chunk(session, file->name, len) == -1) {
+			}
+
+			len = strlen(file->name);
+
+			if (url_encode(file->name, &link) == -1) {
 				break;
 			}
+			if (link == NULL) {
+				if (send_chunk(session, file->name, len) == -1) {
+					break;
+				}
+			} else {
+				if (send_chunk(session, link, strlen(link)) == -1) {
+					free(link);
+					break;
+				}
+				free(link);
+			}
+
 			if (file->is_dir) {
 				if (send_chunk(session, "/", 1) == -1) {
 					break;
 				}
 			}
+
 			if (send_chunk(session, "\">", 2) == -1) {
 				break;
 			} else if (send_chunk(session, file->name, len) == -1) {
 				break;
 			}
+
 			if (file->is_dir) {
 				if (send_chunk(session, "/", 1) == -1) {
 					break;
