@@ -189,7 +189,7 @@ void init_xslt_module() {
 }
 
 bool can_transform_with_xslt(t_session *session) {
-	char *ext, *xslt, *slash;
+	char *xslt, *slash;
 	size_t len;
 	FILE *fp;
 
@@ -201,9 +201,7 @@ bool can_transform_with_xslt(t_session *session) {
 
 	/* Check extension
 	 */
-	if ((ext = file_extension(session->file_on_disk)) == NULL) {
-		return false;
-	} else if (strcmp(ext, "xml") != 0) {
+	if (strcmp(session->extension, "xml") != 0) {
 		return false;
 	}
 
@@ -295,10 +293,10 @@ int transform_xml(t_session *session) {
 
 	/* Mimetype
 	 */
-	session->mimetype = get_mimetype(session->file_on_disk, session->config->mimetype);
+	session->mimetype = get_mimetype(session->extension, session->config->mimetype);
 	if (xslt->method != NULL) {
 		if (strcmp((char*)xslt->method, "html") == 0) {
-			session->mimetype = get_mimetype(".html", session->config->mimetype);
+			session->mimetype = get_mimetype("html", session->config->mimetype);
 		}
 	}
 
